@@ -45,6 +45,7 @@ STAGE_INSTALL_PYPI_PACKAGE = "INSTALL-PYPI-PACKAGE"
 STAGE_PUBLISH_PACKAGE = "PUBLISH-PACKAGE"
 STAGE_PUBLISH_PYPI_PACKAGE = "PUBLISH-PYPI-PACKAGE"
 STAGE_TEST = "TEST"
+STAGE_ANALYZE = "ANALYZE"
 STAGE_TEARDOWN = "TEARDOWN"
 STAGE_PREPARE = "PREPARE"
 
@@ -93,6 +94,9 @@ PUBLISH_PYPI_PACKAGE_SCRIPT_TEMPLATE = 'publish-pypi-package.template.sh'
 TEST_APP_SCRIPT_FILE_PATH = '{bin_dir}/test-app.sh'
 TEST_APP_SCRIPT_TEMPLATE = 'test-app.template.sh'
 
+ANALYZE_APP_SCRIPT_FILE_PATH = '{bin_dir}/analyze-app.sh'
+ANALYZE_APP_SCRIPT_TEMPLATE = 'analyze-app.template.sh'
+
 BEACON_INTERVAL = 60  # seconds
 
 VarStatus = collections.namedtuple("VarInfo", "source_name description target_name")
@@ -115,13 +119,15 @@ predefined_env_variables = None
 default_setup = {
     "docker_image_make_package": "accso/docker-python-app:latest",
     "docker_image_test": "accso/docker-python-app:latest",
-    "docker_image_docker": "marcusrickert/docker-docker-ci:release-0.9",
+    "docker_image_docker": "marcusrickert/docker-docker-ci:release-0.9.1",
+    "docker_image_analyze": "accso/docker-python-app:latest",
     "ci_toolbox_script": "ci_toolbox.py",
     "ci_stage_build_package": STAGE_BUILD_PACKAGE,
     "ci_stage_build_docker_images": STAGE_BUILD_DOCKER_IMAGES,
     "ci_stage_install": STAGE_INSTALL,
     "ci_stage_install_pypi_package": STAGE_INSTALL_PYPI_PACKAGE,
     "ci_stage_test": STAGE_TEST,
+    "ci_stage_analyze": STAGE_ANALYZE,
     "ci_stage_teardown": STAGE_TEARDOWN,
     "ci_stage_publish_package": STAGE_PUBLISH_PACKAGE,
     "ci_stage_publish_pypi_package": STAGE_PUBLISH_PYPI_PACKAGE,
@@ -129,6 +135,7 @@ default_setup = {
     "bin_dir": "bin",
     "test_dir": "test",
     "run_test_suite": "run_{module_name}_test_suite.py",
+    "run_test_suite_no_venv": "run_{module_name}_test_suite_no_venv.py",
     "contrib_dir": "contrib",
     "rel_tmp_dir": "tmp",
     "rel_etc_dir": "etc/{name}",
@@ -177,9 +184,13 @@ default_setup = {
     "babel_rel_directory": None,
     "generate_generic_install": False,
     "max_cpus": None,
+    "analyze": False,
+    "analyze_extra_coverage_exclusions": None,
+    "analyze_extra_exclusions": None,
+    "script_timeout": 600,
 }
 
-logger = None
+logger:logging.Logger = None
 
 
 def get_module_dir(p_module):
@@ -669,6 +680,37 @@ def generate_test_app_script(p_main_setup_module, p_template_env, p_arguments):
 
     os.chmod(test_app_script_filename, stat.S_IXUSR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXGRP | stat.S_IRGRP)
 
+def generate_analyze_app_script(p_main_setup_module, p_template_env, p_arguments):
+    global logger
+
+    fmt = "Generate analyze-app.sh script file for version {version} of app '{name}'"
+    logger.info(fmt.format(**p_main_setup_module.extended_setup_params))
+
+    template = p_template_env.get_template(ANALYZE_APP_SCRIPT_TEMPLATE)
+
+    var = get_vars(p_setup_params=p_main_setup_module.extended_setup_params)
+
+    output_text = template.render(
+        var=var,
+        python_packages=get_python_packages(p_main_setup_module=p_main_setup_module, p_arguments=p_arguments),
+        arguments=p_arguments,
+        site_packages_dir=get_site_packages_dir()
+    )
+
+    output_file_path = ANALYZE_APP_SCRIPT_FILE_PATH.format(**(var["setup"]))
+
+    analyze_app_script_filename = os.path.join(get_module_dir(p_module=p_main_setup_module), output_file_path)
+
+    os.makedirs(os.path.dirname(analyze_app_script_filename), mode=0o777, exist_ok=True)
+
+    with open(analyze_app_script_filename, "w") as f:
+        f.write(output_text)
+
+    fmt = "Wrote analyze-app.sh script file to '{filename}'"
+    logger.info(fmt.format(filename=analyze_app_script_filename))
+
+    os.chmod(analyze_app_script_filename, stat.S_IXUSR | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXGRP | stat.S_IRGRP)
+
 
 def generate_publish_debian_package_script(p_main_setup_module, p_template_env, p_arguments):
     global logger
@@ -750,37 +792,35 @@ def execute_generated_script(p_main_setup_module, p_script_file_path_pattern):
 
     script_filename = os.path.join(get_module_dir(p_module=p_main_setup_module), output_file_path)
 
-    fmt = "<<<<< START script {filename} ..."
-    logger.info(fmt.format(filename=script_filename))
+    timeout = p_main_setup_module.extended_setup_params.get("script_timeout")
+
+    fmt = "<<<<< START script {filename} (timeout={timeout}[sec])..."
+    logger.info(fmt.format(filename=script_filename, timeout=timeout))
 
     extended_env = os.environ.copy()
     extended_env.update(get_predefined_environment_variables())
 
     expand_vars(extended_env)
+    thread = None
+    status = None
 
     try:
-        popen = subprocess.Popen(script_filename, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=extended_env)
-
         status = tools.SimpleStatus()
         thread = tools.start_simple_thread(output_beacon, status)
 
-        stdout, stderr = popen.communicate()
+        process = subprocess.Popen(script_filename, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
+                                   env=extended_env, bufsize=1, universal_newlines=True)
 
-        status.done = True
-        thread.join()
+        msg = "[STDOUT/STDERR] {line}"
 
-        exit_code = popen.returncode
-        msg = "[STDOUT] {line}"
+        for output in iter(process.stdout.readline, ""):
+            for line in output.split("\n"):
+                if line != '':
+                    logger.info(msg.format(line=line))
 
-        for line in stdout.decode("utf-8").split("\n"):
-            if line != '':
-                logger.info(msg.format(line=line))
+        process.communicate(timeout=5)
+        exit_code = process.returncode
 
-        msg = "[STDERR] {line}"
-
-        for line in stderr.decode("utf-8").split("\n"):
-            if line != '':
-                logger.info(msg.format(line=line))
 
     except subprocess.CalledProcessError as e:
         exit_code = e.returncode
@@ -789,7 +829,17 @@ def execute_generated_script(p_main_setup_module, p_script_file_path_pattern):
 
     except Exception as e:
         exit_code = -1
-        logger.error("General exception in subprocess!")
+        msg = "General exception '{exception}' in subprocess!"
+        logger.error(msg.format(exception=str(e)))
+
+    finally:
+
+        if status is not None:
+            status.done = True
+
+        if thread is not None:
+            thread.join()
+
 
     msg = ">>>>> END script {filename} ..."
     logger.info(msg.format(filename=script_filename))
@@ -832,6 +882,9 @@ def execute_test_app_script(p_main_setup_module):
     execute_generated_script(p_main_setup_module=p_main_setup_module,
                              p_script_file_path_pattern=TEST_APP_SCRIPT_FILE_PATH)
 
+def execute_analyze_app_script(p_main_setup_module):
+    execute_generated_script(p_main_setup_module=p_main_setup_module,
+                             p_script_file_path_pattern=ANALYZE_APP_SCRIPT_FILE_PATH)
 
 def get_parser():
     parser = argparse.ArgumentParser()
@@ -844,6 +897,7 @@ def get_parser():
                                  STAGE_INSTALL,
                                  STAGE_INSTALL_PYPI_PACKAGE,
                                  STAGE_TEST,
+                                 STAGE_ANALYZE,
                                  STAGE_TEARDOWN,
                                  STAGE_PREPARE])
     parser.add_argument('--run-dir', dest='run_dir', default=None)
@@ -914,6 +968,11 @@ def main(p_main_module_dir):
                                   p_arguments=arguments)
             execute_test_app_script(p_main_setup_module=main_setup_module)
 
+        elif arguments.execute_stage == STAGE_ANALYZE:
+            generate_analyze_app_script(p_main_setup_module=main_setup_module, p_template_env=template_env,
+                                     p_arguments=arguments)
+            execute_analyze_app_script(p_main_setup_module=main_setup_module)
+
         elif arguments.execute_stage == STAGE_PREPARE:
             generate_gitlab_ci_configuration(p_main_setup_module=main_setup_module, p_template_env=template_env)
             generate_circle_ci_configuration(p_main_setup_module=main_setup_module, p_template_env=template_env)
